@@ -1,10 +1,10 @@
 use alloc::{collections::VecDeque, sync::Arc};
 use parking_lot::Mutex;
 
-type Inner<T> = Arc<Mutex<VecDeque<T>>>;
+type Inner<T> = Arc<Mutex<Option<VecDeque<T>>>>;
 
 pub fn create_queue<T>() -> (Sender<T>, Receiver<T>) {
-    let inner = Arc::new(Mutex::new(VecDeque::new()));
+    let inner = Arc::new(Mutex::new(Some(VecDeque::new())));
     let sender = Sender(inner.clone());
     let receiver = Receiver {
         inner,
@@ -24,7 +24,9 @@ impl<T> Clone for Sender<T> {
 
 impl<T> Sender<T> {
     pub fn send(&self, value: T) {
-        self.0.lock().push_back(value)
+        if let Some(queue) = self.0.lock().as_mut() {
+            queue.push_back(value);
+        }
     }
 }
 
@@ -37,9 +39,21 @@ pub struct Receiver<T> {
 impl<T> Receiver<T> {
     pub fn drain(&mut self) -> alloc::collections::vec_deque::Drain<T> {
         if self.drain.is_empty() {
-            core::mem::swap(&mut *self.inner.lock(), &mut self.drain);
+            if let Some(queue) = self.inner.lock().as_mut() {
+                core::mem::swap(queue, &mut self.drain);
+            }
         }
 
         self.drain.drain(..)
     }
+
+    pub fn close(&mut self) -> Option<VecDeque<T>> {
+        self.inner.lock().take()
+    }
+}
+
+impl<T> Drop for Receiver<T> {
+    fn drop(&mut self) {
+        self.close();
+    }
 }
